subroutine PTthermo_props2(ncp, max, status11, TK, rhox, m, x, u0, e_ii, kij, N_sites, fa, &
						& fb, kappa, sig, D, v00, Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, chpt, &
						& Pcalc, fugacity, chpt_res, Gtotal, PC_dispersion) 


!The only difference between this version and the regular version is that this version has a print statement for
! prsm
!	Program by P. David Ting from codes by Walter G. Chapman
!
!	Purpose:
!		Driver program that calculates the thermodynamic contributions of hard sphere
!		chains (made up of mean field/dispersion contributions, hard sphere contributions, 
!		chain formation contributions, and association contributions).  The theory is due to
!		Wertheim and much of the codes are  obtained from W.G. Chapman.  This subroutine is 
!		valid for any number of components.       
!
!	This code calculates:
!		residual pressure [P/rho kT]
!		residual internal energy [U/NkT]
!		residual Helmholtz free energy [A/NkT]
!		residual Gibbs free energy [G/NkT]
!		residual entropy [S/Nk]
!		residual enthalpy [H/NkT]
!		residual chemical potential of component i [chptl/kT]
!		natural log of fugacity coefficient [ln phi(i)]
!
!
!	This routine needs the following subroutines:
!		dispersion.f90							Q_sub.f90
!		ghs_derivatives.f90						zeta_s.f90
!		hard_chain_eos.f90						hs_cs.f90
!		hbond_thermo.f90
!
!	Record of revisions:
!			Date		Programmer				Description of change
!			====		==========				=====================
!			1997		W.G. Chapman			Thermoprop subroutines
!			7/29/99		P.D. Ting				Revised for multicomponent and updated 
!													for fortran 90/95	
implicit none

!	Checking single & double precision
integer, parameter :: single = SELECTED_REAL_KIND(p=6, r=37)
integer, parameter :: double = SELECTED_REAL_KIND(p=13)

!	Declaring calling variables
integer, intent(in) :: ncp									! declaring number of components
integer, intent(in) :: max									! max number of sites on all molecules
integer, intent(in) :: status11								! HBond on/off trigger
integer, intent(in) :: PC_dispersion						! use PC or CK dispersion

real (KIND=double), intent(in) :: TK						! Temperature in Kelvin
real (KIND=double), intent(in) :: rhox						! number density of molecules (per ang^3)
real (KIND=double), intent(in), dimension(ncp) :: m			! chain length of molecule i
real (KIND=double), intent(in), dimension(ncp) :: x			! mole fraction of component i
real (KIND=double), intent(in), dimension(ncp, ncp) :: u0	! u0_i,i)/k in Kelvin
real (KIND=double), intent(in), dimension(ncp) :: v00	! temp independent volume element
real (KIND=double), intent(in), dimension(ncp) :: e_ii		! e(i,i) used in dispersion calculation 
real (KIND=double), intent(in), dimension(ncp, ncp) :: kij	! binary kij used in dispersion
integer, intent(in), dimension(ncp) :: N_sites				! number of bonding sites on each specie
real (KIND=double), intent(in), dimension(ncp, ncp, max, max,2) :: fa	! association epsilon(AiBj) &
																		! its mayer f function
real (KIND=double), intent(in), dimension(ncp, ncp, max, max) :: kappa	! association kappa(AiBj)
real (KIND=double), intent(in), dimension(ncp, ncp, max, max, 1) :: fb	! d(fa) / dT
real (KIND=double), intent(in), dimension(ncp, ncp) :: sig				! temp independent sphere diameter
real (KIND=double), intent(in), dimension(4,9) :: D						! Chen & Kreglewski dispersion parameter

real (KIND=double), intent(out) :: Prsm						! total residual P
real (KIND=double), intent(out) :: Pcalc					! calculated Pressure in bars
real (KIND=double), intent(out) :: Ursm						! total residual U
real (KIND=double), intent(out) :: Arsm						! total residual A
real (KIND=double), intent(out) :: Grsm						! total residual G
real (KIND=double), intent(out) :: Srsm						! total residual S
real (KIND=double), intent(out) :: Hrsm						! total residual H
real (KIND=double), intent(out) :: Gtotal					! Total Gibbs free energy of mixture
real (KIND=double), intent(out), dimension(ncp) :: chpt		! total chemical potential
real (KIND=double), intent(out), dimension(ncp) :: chpt_res	! residual total chemical potential
real (KIND=double), intent(out), dimension(ncp) :: fugacity	! natural log of fugacity coefficient


!	Declaring variables used in the subroutines
real (KIND=double) :: rho_spheres						! density of spheres (# spheres/volume in A^3)
real (KIND=double), dimension(ncp) :: dhs				! temp. dependent diameter
real (KIND=double), dimension(ncp) :: sitos1			! temp. dependent reduced diameter
real (KIND=double), dimension(ncp) :: x_spheres			! x(i)*m(i)/sum[x(i)*m(i)] mole fraction of spheres
real (KIND=double), dimension(ncp, ncp) :: c			! variable lump used in calculations
real (KIND=double), dimension(ncp, ncp) :: dcdt			! derivative of c wrt T
real (KIND=double), dimension(ncp) :: dddt				! derivative of dhs wrt T reduced by sig(1,1)
real (KIND=double) :: zeta0
real (KIND=double) :: zeta1
real (KIND=double) :: zeta2
real (KIND=double) :: zeta3
real (KIND=double) :: dz1dt								! derivative of zeta1 wrt T
real (KIND=double) :: dz2dt								! derivative of zeta2 wrt T
real (KIND=double) :: dz3dt								! derivative of zeta3 wrt T
real (KIND=double), dimension(ncp, ncp) :: g			! hard sphere g(r) at contact
real (KIND=double), dimension(ncp, ncp) :: rhodgdrho	! rho * [dg(i,j)/drho]
real (KIND=double), dimension(ncp, ncp, ncp) :: dgdrho	! dg(i,j) / drho(m)
real (KIND=double), dimension(ncp, ncp) ::dgdt			! dg(i,j) / dT
real (KIND=double) :: pcref								! conf. pressure from reference fluid
real (KIND=double) :: Arref								! refererence residual A
real (KIND=double) :: Prref								! refererence residual P
real (KIND=double) :: Hrref								! refererence residual H
real (KIND=double) :: Urref								! refererence residual U
real (KIND=double) :: Srref								! refererence residual S
real (KIND=double) :: Grref								! refererence residual G
real (KIND=double), dimension(ncp) :: chptref			! reference res. chemical potential 
real (KIND=double) :: Amf								! mean field residual A
real (KIND=double) :: Pmf								! mean field residual P
real (KIND=double) :: Hmf								! mean field residual H
real (KIND=double) :: Umf								! mean field residual U
real (KIND=double) :: Smf								! mean field residual S
real (KIND=double) :: Gmf								! mean field residual G
real (KIND=double), dimension(ncp) :: chptlmf			! mean field res. chemical potential 
real (KIND=double) :: Ahc								! hard chain residual A
real (KIND=double) :: Phc								! hard chain residual P
real (KIND=double) :: Hhc								! hard chain residual H
real (KIND=double) :: Uhc								! hard chain residual U
real (KIND=double) :: Shc								! hard chain residual S
real (KIND=double) :: Ghc								! hard chain residual G
real (KIND=double), dimension(ncp) :: chpthc			! hard chain res. chemical potential 
real (KIND=double) :: Ahb								! association residual A
real (KIND=double) :: Phb								! association residual P
real (KIND=double) :: Hhb								! association residual H
real (KIND=double) :: Uhb								! association residual U
real (KIND=double) :: Shb								! association residual S
real (KIND=double) :: Ghb								! association residual G
real (KIND=double), dimension(ncp) :: chptlhb			! association res. chemical potential 


!	Declaring parameters
real (KIND=double) :: AVOG = 0.602252d0				! Nav / (10^8)^3
real (KIND=double) :: R = 83.1451					! cm3 bar/mol /k

!	Declaring local variables
integer :: i, j, a, b, k, l									! index counter
real (KIND=double) :: ratio									! ratio between segments (spheres) & chains

!write(6,*) 'x',x
call test_zeta_s(sig, u0, rho_spheres, x, x_spheres, sitos1, zeta0, zeta1, zeta2, &
			& zeta3, dz1dt, dz2dt, dz3dt, m, dhs, ncp, TK, rhox, c, dcdt, dddt)

!write(6,*) 'zeta_0', zeta0
!write(6,*) 'zeta_1', zeta1
!write(6,*) 'zeta_2', zeta2
!write(6,*) 'zeta_3', zeta3
!write(6,*) 'dhs', dhs



call test_ghs_derivatives(sitos1, zeta2, zeta3, m, ncp, c, dcdt, dz1dt, dz2dt, dz3dt, g, &
			& rhodgdrho, dgdt, dgdrho)

call test_hs_cs(sitos1, x_spheres, ncp, rho_spheres, dz1dt, dz2dt, dz3dt, TK, Prref, Arref, &
			& Grref, Srref, Urref, Hrref, chptref, pcref)


if (PC_dispersion.eq.0) then
	! this invokes the CK dispersion type terms. For more details, see Haung and Rasdoc, 1990 IECR
	call test_dispersion(D, ncp, e_ii, TK, u0, sitos1, kij, x, m, rhox, zeta3, Amf, Pmf, Gmf, &
			& Hmf, Smf, Umf, chptlmf, dddt, dz3dt)
	
	elseif (PC_dispersion.eq.1) then 
		
		call test_PCdispersion(ncp, sig, u0, TK, kij, rhox, zeta3, Amf, Pmf, Gmf, &
				& Hmf, Smf, Umf, chptlmf, x, m, dhs)
	
	else
		write(6,*) 'PC_dispersion flag in input parameters needs to be set to 0 or 1'
		! PAUSE
		STOP
end if

call test_hard_chain_eos(rhox, x, m, ncp, TK, sitos1, g, chpthc, Phc, Ahc, Ghc, Uhc, Shc, Hhc, &
			& rhodgdrho, dgdrho, dgdt)

if (status11 .eq. 0) then
	Phb = 0.0
	Uhb = 0.0
	Ahb = 0.0
	Ghb = 0.0
	Shb = 0.0
	Hhb = 0.0
	forall (i=1:ncp)
		chptlhb(i) = 0.0
	end forall
	
	else

		call test_hbond_thermo2(ncp, max, rhox, N_sites, g, dgdrho, sig, fa, kappa, fb, dgdt, &
			& rhodgdrho, Phb, Uhb, Ahb, Ghb, Shb, Hhb, chptlhb, x, TK) 
end if






!	Calculating residual properties
ratio = rho_spheres / rhox

Prsm = Prref * ratio + Pmf + Phc + Phb
Ursm = Urref * ratio + Umf + Uhc + Uhb
Arsm = Arref * ratio + Amf + Ahc + Ahb
Srsm = Srref * ratio + Smf + Shc + Shb
Hrsm = Hrref * ratio + Hmf + Hhc + Hhb
Grsm = Grref * ratio + Gmf + Ghc + Ghb

forall (i=1:ncp)
	chpt_res(i) = m(i) * chptref(i) + chptlmf(i) + chpthc(i) + chptlhb(i)
end forall

do i = 1, ncp
	if (x(i) .gt. 0.0d0) then
		chpt(i) = chpt_res(i) + log(x(i) * rhox)					! true chemical potential
	end if

end do



if (Prsm .gt. -1.0d0) then

	forall (i=1:ncp)
		fugacity(i) = chpt_res(i) - log(Prsm + 1.0d0) !- log(m(i))		! natural log of fugacity
																		! coefficient
	end forall

	!Gtotal = (grsm - log(Prsm + 1.0d0)) * 8.314* TK					! residual G wrt const T,P
! This block of code modified by david ting, added 03-21-02)    
	Gtotal = 0.0d0
	do i = 1, ncp
	  Gtotal = Gtotal + x(i) * chpt(i)
	end do
	Gtotal = Gtotal * R * TK
! !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 

	Pcalc = (Prsm + 1.0d0)*rhox*TK / ( AVOG * sig(1,1)**3 ) * R

else 
  
    Pcalc = (Prsm + 1.0d0)*rhox*TK / ( AVOG * sig(1,1)**3 ) * R  !Pcalc = 69
end if
! write(6,*) 'Prsm', prsm

!write(6,*) 'rhox in thermoprops', rhox
end subroutine
