Program main_driver

!	This is the driver that converts the constant T & rho SAFT to a constant P & T SAFT.
!	This program should be run on top of thermo_props.  It uses dneqnf root finding
!	routine in IMSL to solve PTfcn (which varies composition to reach the desired pressure). 
!	Note that temperature, pressure, and compositions are fixed with density being the variable.
!
!	This code needs the following subroutines
!
!	Calls the IMSL routine: DNEQNF which solves a system of nonlinear eqns using a modified
!		Powell hybrid algorithm and a finite difference approx. to the Jacobian.  
!		See IMSL Help for usage.
!
!	Calls the IMSL routine DUMINF which minimizes a given function using a quasi-Newton method
!		and a finite-difference gradient.
!
!	Record of revisions:
!			Date		Programmer				Description of Change
!			====		==========				=====================
!			3/31/00 	P.D. Ting				inception
!			7/09/02		M. Yarrison				Added PC dispersion ability, modified interface 
!												
use essentials
implicit none

!	Declaring local (thermo_props) variables
integer :: i, j, k,l,a, b, iersvr, ipact, isact, pick, itmax, N, Np
real (KIND=double) :: rhox						! number density of molecules (per ang^3)
real (KIND=double) :: errel			            ! Relative error for nonlinear eqn. solver
real (KIND=double), dimension(200, 50) :: ycomp ! Composition of the second phase in T-X flash
real (KIND=double), dimension(200)     :: Pressure ! Pressures calculated by T-X flash
real (KIND=double), dimension(200)     :: fnorm1 ! fnorm from nonlinear equation solver
real (KIND=double), dimension(200, 50) :: pvap ! Pure component vapor pressures
real (KIND=double)  :: temp
real (KIND=double), dimension(50) :: f 

real (KIND=double), allocatable, dimension(:) :: pbb
real (KIND=double) :: ybb, fvalue_uminf			! trial unstable phase mole fraction
real (KIND=double), allocatable, dimension(:) :: fugacity_z	! feed fugacity
real (KIND=double), allocatable, dimension(:) :: x
real (KIND=double), allocatable, dimension(:) :: phaseX, phaseY, tempcomp
real (KIND=double) :: rhoxphaseX, rhoxphaseY
real (KIND=double), dimension(51) :: den, denf, denp, denpf	! used in dneqnf
real (kind=double), allocatable, dimension(:) :: liquiddens, vapordens, crap
!	Declaring local parameters
real (KIND=double) :: AVOG = 0.602252d0 		! Nav / (10^8)^3
real (KIND=double) :: R = 83.1451				! cm3 bar/mol /k
real (KIND=double) :: pi = 3.141592654d0
real (KIND=double) :: tau = 0.7404804897

!	Declaring IMSL variables
real (KIND=double), dimension(7) :: rparam2
integer, dimension(7) :: iparam2
integer :: counter, phase
real (KIND=double) :: limit
real (KIND=double) :: step, bound, xacc, fnorm
integer :: maxfn
real (KIND=double) :: kij_mini_uvmif

!	Declaring other local variables

real (KIND=double) :: sum, sum2
real (KIND=double) :: kij_guess12, kij_result12

external kij_mini_uvmif, duvmif, user_interface, chenkreglewski_disp, &
		& liquid_or_vapor, driver_michelsen, driver_phasecalc, funcv, dneqnf

!write(6,*) 'What system do you wish to model?'
!write(6,*) 'enter 0 to enter parameters from a file'
!read(5,*), system ! also declared in Essentials
!write(6,*), system
!write(6,*) ' '

!call input_parameters

!call qd_user_interface(pick)

call user_interface(pick)
if (pc_dispersion.eq.0) then 
	call chenkreglewski_disp
	else
	write(6,*) 'PC dispersion will be used for this system'
end if

allocate (pbb(ncp), fugacity_z(ncp), x(ncp), phaseX(ncp), phaseY(ncp), tempcomp(ncp), liquiddens(npts), vapordens(npts), crap(npts))


!	Conditioning the IMSL parameters
step = 0.001
bound = 0.3
xacc = 1.0d-6
errel = 1.0d-11
itmax = 500000
maxfn = 500000
iersvr = 0
ipact = 1
isact= 0 
call DU4INF(iparam2, rparam2)
iparam2(3) = 100000
iparam2(4) = 100000
iparam2(6) = 0

!call erset(iersvr, ipact, isact)

write(6,*) 'Please wait while we compute the solutions'

!	Checks whether the input condition results in a vapor phase or a liquid phase
!	The resulting density of the lower gibbs free energy phase will be used

if (pick.eq.1) then
! The PX code (pick =3) will go here 
  forall (i=1:ncp)
    x(i) = x_tmp(i)
  end forall

  call liquid_or_vapor(rhox, x)
  rhox_tmp = rhox


!	Check this input phase's stability
  call driver_michelsen(rhox, pbb, ybb, fvalue_uminf, iparam2, rparam2, fugacity_z)
  if (fvalue_uminf.gt.-1.0d-17) then
    call driver_stability2(ybb, pbb, iparam2, rparam2)
  end if


!	If unstable, we will use a form of Rachford-Rice to find the equilibrium
!	phase composition.
  write(6,*) 'ybb', ybb
  write(6,*) 'pbb', pbb

  if (ybb.le.-1.0d-17) then
    do phase = 1,2

      call driver_phasecalc(pbb, fugacity_z, phase, limit, counter, phaseX, phaseY, &
				&	rhoxphaseX, rhoxphaseY)

	  if ((limit.lt.1.0d-6).and.(counter.lt.6000)) then
	   write(6,*) '2 Phase region'
	   exit
	  end if
	  if ((phase.eq.2).and.(limit.gt.1.0d-6).and.(counter.eq.6000)) write(6,*) 'no solution'
    
	end do

    if(rhoxphaseX.lt.rhoxphaseY) then
	  temp=rhoxphaseX
	  rhoxphaseX=rhoxphaseY
	  rhoxphaseY=temp
	  tempcomp=phaseX
	  phaseX=phaseY
	  phaseY=tempcomp
	end if

	else if(abs((rhoxphasex-rhoxphasey)).lt.1.0d-7) then
	write(6,*) 'this P,T and x composition point appears to be a'
	write(6,*) ' stable 1 phase system. Please alter the P,T or X' 
!		pause
	STOP
	
	else 
	    write(6,*) 'The provided species composition, temperature, and pressure'
		write(6,*) 'seems to be stable as a one phase system.'
		write(6,*) 'Pick a different thermodynamic state point and try again :)'
!		Pause
		STOP

  end if

end if	


!	Runs the constant composition & temperature code.

if (pick.eq.1) then

 den(1)=rhoxphaseX
 den(2)=rhoxphaseY
 
 
 write(6,*) 'den(1) is',den(1)
 write(6,*) 'den(2) is',den(2)
	 if(abs(rhoxphasex-rhoxphasey).lt.1.0e-7) then
		write(6,*) 'this P,T and x composition point appears to be a'
		write(6,*) ' stable 1 phase system. Please alter the P,T or X' 
!	    pause
		STOP
	  end if	
 
!pause
 do i=1,ncp-1
  den(2+i)=phaseY(i)
 end do
 do i=1,npts
   TK=TCA(i)	
   forall (j=1:ncp)
    xliq(j)=xcomp(i,j)
   end forall
   N=ncp+1    ! Number of equations
   call dneqnf(funcv, errel, N, itmax, den, denf, fnorm) 
   do k=1,N
    den(k)=denf(k)
	!write(6,*) 'assigning densities in main_driver'	
   end do
	! This assigns the calculated phase densities so they can be output 
	liquiddens(i) = den(1)
	vapordens(i) = den(2)   
    crap(i) = fnorm
   Pressure(i) = PL
   
   sum=0.0d0
   if(purefluid) then
    forall (j=1:ncp)
	  ycomp(i,j)=0.d0
	end forall
	ycomp(i,ipure)=1.d0
   else
    do j=1,ncp-1
     ycomp(i,j)=den(2+j)
     sum=sum+ycomp(i,j)
    end do 
    ycomp(i,ncp)=1.0d0-sum
   end if
   fnorm1(i)=fnorm 
 
   !write(6,*) 'density', den
   !write(6,*) ' '
 end do
 
endif



!	The code below runs the data fitting routines
if (pick.eq.2) then
! ADJUST THIS TO CONVERGE FASTER!!!!!
  kij_guess12 = 0.01
  step=0.005	
  call DUVMIF(kij_mini_uvmif, kij_guess12, step, bound, xacc, maxfn, kij_result12)


  write(6,*) ' ' 
  write(6,*) 'Fitted optimal kij value', kij_result12

  sum = kij_mini_uvmif(kij_result12)
  write(6,*) 'Fitted optimal residual', sum
pause	
end if



!	Convert mole fraction back to mass fraction if polymer=2 is chosen

if (polymer .eq. 2) then

  sum2 = 0.0d0
  sum = 0.0d0
  do i = 1, ncp
    sum = sum + phaseX(i)*mw_tmp(i)
    sum2 = sum2 + phaseY(i)*mw_tmp(i)
  end do

  forall (i=1:ncp-1)
    phaseX(i) = phaseX(i)*mw_tmp(i)/sum
    phaseY(i) = phaseY(i)*mw_tmp(i)/sum2
  end forall

  sum = 0.0d0
  sum2 = 0.0d0
  do i = 1, ncp-1
    sum = sum + phaseX(i)
    sum2 = sum2 + phaseY(i)
  end do
  phaseX(ncp) = 1.0d0 - sum
  phaseY(ncp) = 1.0d0 - sum2 



  do i = 1, npts
    sum=0.d0
    sum2=0.d0
!
    do k = 1, ncp
       sum = sum + xcomp(i,k)*mw_tmp(k)
       sum2 = sum2 + ycomp(i,k)*mw_tmp(k)
    end do
!
   
    forall (k=1:ncp)
      xcomp(i,k) = xcomp(i,k)*mw_tmp(k)/sum
      ycomp(i,k) = ycomp(i,k)*mw_tmp(k)/sum2
    end forall

  end do
  
end if

write(6,*) 'phaseX', phaseX
write(6,*) 'phaseY', phaseY
write(6,*) ' '
write(6,*) 'specified points below:'	
write(6,*) '   T(K)	  P(bar)   x(ncp-1)   y(ncp-1)   fnorm'
do i = 1, npts
  write(6,54)TCA(i),Pressure(i),(xcomp(i,j),j=1,ncp-1),(ycomp(i,j),j=1,ncp-1), fnorm1(i)
end do
54	format(F10.2, 2x,F12.5, 2x, 2(F10.5,2x),e12.4) 

OPEN (Unit=12, FILE="Txoutput.txt",STATUS="unknown", ACTION="READWRITE", POSITION="APPEND")
	 !do i = 1 , ncp
	! write (12, fmt = '(7 C14, 2x)'),  phaseX(
	do i=1,npts
		write (12, fmt = '( 6 ( F12.7, 2x ), f10.2, 3x, f12.6, 2x, e12.5, 2x, f12.4 ) ') ,xcomp(i,1), ycomp(i,1), xcomp(i,2), ycomp(i,2), liquiddens(i), vapordens(i), Tca(i), Pressure(i), crap(i), kij_tmp(1,2)
	end do
	
 CLOSE ( unit=12,status="keep")

pause

END PROGRAM