SUBROUTINE two_phase_flash(flag3, flag4, phaseY, phaseX, fugacityY, fugacityX, limit)

!	This is the two phase flash routine.  The routine uses a Rachford-Rice formulation to
!	calculate the phase equilibrium of the two phases.
!

use essentials
implicit none

!	Input/Output argument list
real (KIND=double), dimension(ncp) :: phaseY, phaseX, fugacityY, fugacityX
integer, intent(in) :: flag3, flag4
real (KIND=double) :: limit

!	Internal working variables
integer :: i, j
real (KIND=double) sum, sum2
integer, dimension(ncp) :: array, array2

!	IF we have both phases present in the system.				
sum = 0.0d0 
sum2 = 0.0d0

if ((flag3 .eq. 1) .and. (flag4 .eq. 1) ) then	
					
  do i = 1, ncp
	sum = sum + abs((log(phaseY(i)) + fugacityY(i) - log(phaseX(i)) - fugacityX(i)))
		
!	Normal damping factor is 0.15.	It should be set lower with lower pressure		
!	mole fraction based partition function calculations

	partition(i) = log(partition(i))-0.1*(log(phaseY(i))+fugacityY(i)-log(phaseX(i)) &
				& - fugacityX(i) ) 
	partition(i) = exp(partition(i))

  end do

  limit = sum/ncp
end if

!	We will first deal with vapor side (or the label Y).  If components are absence in phase Y
!	(or phase 2) but all components are present in phase X (or phase 1), we do the following:

if ((flag3 .eq. 0) .and. (flag4 .eq. 1) ) then
				
  forall(i = 1:ncp)
	array(i) = 1
  end forall

  do i=1,ncp
	if (phaseY(i) .lt. 1.0d-25) then			! Indexing component presence 
	  array(i) = 0
	  phaseY(i) = 0.0d0
	  partition(i) = 0.0d0
	end if
  end do

  sum = 0.0d0
	
  do i = 1, ncp									! Tallying up stop criteria
	if (array(i) .gt. 0) then			
	  sum = sum + abs(log(phaseY(i)) + fugacityY(i) - log(phaseX(i)) - &
			& fugacityX(i))
	end if					
  end do
  limit = sum/ncp

  do i = 1, ncp									! Calculating properties for 
	if (array(i) .gt. 0) then					! components that are present
	  partition(i) = log(partition(i))-0.1*(log(phaseY(i))+fugacityY(i)- & 
				& log(phaseX(i)) - fugacityX(i) ) 

	  partition(i) = exp(partition(i))
	end if	
  end do
end if

!	We will now deal with liquid side (or the label X).  If components are absence in phase X 
!	(or phase 1) but all components are present in phase Y (or phase2), we do the following:
!	Below is just a flip of the previous sequence of codes.

if ((flag3 .eq. 1) .and. (flag4 .eq. 0)) then

  forall(i = 1:ncp)
	array(i) = 1
  end forall

  do i=1,ncp
	if (phaseX(i) .lt. 1.0d-25) then			! Indexing component presence 
  	  array(i) = 0
	  phaseX(i) = 0.0d0
	  partition(i) = 0.0d0
	end if
  end do

  sum = 0.0d0

  do i = 1, ncp									! Tallying up stop criteria
    if (array(i) .gt. 0) then			
	  sum = sum + abs(log(phaseY(i)) + fugacityY(i) - log(phaseX(i)) - fugacityX(i))
	end if					
  end do
  limit = sum/ncp

  do i = 1, ncp									! Calculating properties for 
	if (array(i) .gt. 0) then					! components that are present

 	  partition(i) = log(partition(i))-0.1*(log(phaseY(i))+fugacityY(i)- & 
			& log(phaseX(i)) - fugacityX(i) ) 
	  partition(i) = exp(partition(i))

    end if 				
  end do		
end if


!	We will now deal with mixed cases where the component present in phase X are absence in 
!	phase Y and vice versa.  In this case, we will sieve out the components that are present 
!	in both phases.  At this point, the code cannot deal with partitioning in which components
!	partitioned into one phase is not present in the other phase.  

if ((flag3 .eq. 0) .and. (flag4 .eq. 0) ) then
            
  forall(i = 1:ncp)
	array(i) = 1
    array2(i) = 1
  end forall

  do i = 1, ncp
    if (phaseX(i) .lt. 1.0d-25) then
      array(i) = 0
      phaseX(i) = 0.0d0
      partition(i) = 0.0d0
    end if
    if (phaseY(i) .lt. 1.0d-25) then
      array2(i) = 0
      phaseY(i) = 0
      partition(i) = 0.0d0
    end if
  end do
  sum = 0.0d0
  do i = 1, ncp
    if ((array(i) .gt. 0) .and. (array2(i) .gt. 0)) then            ! Tallying up stop criteria
      sum = sum + abs(log(phaseY(i)) + fugacityY(i) - log(phaseX(i)) - fugacityX(i))
    end if                  
  end do
  limit = sum/ncp

  do i = 1, ncp                                                   ! Calculating properties for
    if ((array(i) .gt. 0) .and. (array2(i) .gt. 0) ) then       ! components that are present

      partition(i) = log(partition(i))-0.1*(log(phaseY(i))+fugacityY(i)- & 
                 & log(phaseX(i)) - fugacityX(i) ) 

      partition(i) = exp(partition(i))
    end if          
  end do      
end if


END SUBROUTINE
